@@ -14,31 +14,6 @@ writeToDisk() on an instance of this class. See __main__ for options.
 See open_edx_export_class/src/exportClass.py for example of using
 this module as a library.
 
-Assumes availability of the following DB table, which is generated 
-via script prepEngagementAnalysis.sh 
-mysql> DESCRIBE Activities;
-
-+---------------------+--------------+------+-----+---------------------+-------+
-| Field               | Type         | Null | Key | Default             | Extra |
-+=====================+==============+======+=====+=====================+=======+
-| course_display_name | varchar(255) | NO   |     |                     |       |
-+---------------------+--------------+------+-----+---------------------+-------+
-| anon_screen_name    | text         | NO   |     | NULL                |       |
-+---------------------+--------------+------+-----+---------------------+-------+
-| event_type          | text         | NO   |     | NULL                |       |
-+---------------------+--------------+------+-----+---------------------+-------+
-| time                | datetime     | NO   |     | 0000-00-00 00:00:00 |       |
-+---------------------+--------------+------+-----+---------------------+-------+
-| isVideo             | tinyint(4)   | NO   |     | 0                   |       |
-+---------------------+--------------+------+-----+---------------------+-------+
-
-Example rows::
-
-| Education/EDUC115N/How_to_Learn_Math | 00014bffc716bf9d8d656d2f668f737cd43acde8 | seek_video | 2013-07-20 22:56:36      | 1 |
-| Education/EDUC115N/How_to_Learn_Math | 00014bffc716bf9d8d656d2f668f737cd43acde8 | hide_transcript | 2013-07-20 22:57:28 | 0 |
-| Education/EDUC115N/How_to_Learn_Math | 00014bffc716bf9d8d656d2f668f737cd43acde8 | play_video | 2013-07-20 22:59:36      | 1 |
-
-
 To use, instantiate EngagementComputer, call the run() method,
 and then the writeResultsToDisk() method.   
 
@@ -99,11 +74,15 @@ sys.path = source_dir
 
 class EngagementComputer(object):
     
-    # Time duration counted for any video event (minutes):
-    VIDEO_EVENT_DURATION = 5
+    # Time duration counted for any video event (minutes);
+    # This is the median time between clicks across all
+    # events:
+    VIDEO_EVENT_DURATION = 504/60
     
-    # Time duration counted for any non-video event (minutes):
-    NON_VIDEO_EVENT_DURATION = 5
+    # Time duration counted for any non-video event (minutes);
+    # This is the median time between clicks across all
+    # events:
+    NON_VIDEO_EVENT_DURATION = 504/60
     
     # Database that contains EventXtract table:
     EVENT_XTRACT_TABLE_DB = 'Edx'
@@ -114,8 +93,6 @@ class EngagementComputer(object):
     def __init__(self, 
                 coursesStartYearsArr, 
                 dbHost, 
-                dbName, 
-                tableName, 
                 mySQLUser=None, 
                 mySQLPwd=None, 
                 courseToProfile=None, 
@@ -130,13 +107,6 @@ class EngagementComputer(object):
         :type coursesStartYearsArr: {[int] | None}
         :param dbHost: MySQL host where the activities table resides 
         :type dbHost: string
-        :param dbName: name of db within server in which the activities table resides. Use 
-            this parameter to place test tables into, say the 'test' database. Point this dbName
-            parameter to 'test' and all ops will look for the Activities table and CourseRuntimes
-            table in that db.
-        :type dbName: string
-        :param tableName: name of table that holds the activities as per file level header. 
-        :type tableName: string
         :param mySQLUser: user under which to log into MySQL for the work
         :type mySQLUser: string
         :param mySQLPwd: password to use for MySQL
@@ -150,8 +120,7 @@ class EngagementComputer(object):
         :type sessionInactivityThreshold: int
         '''
         self.dbHost = dbHost
-        self.dbName = dbName
-        self.tableName = tableName
+        self.dbName = 'Edx'
         self.mySQLUser = mySQLUser
         self.mySQLPwd  = mySQLPwd
         if courseToProfile == "None":
@@ -179,7 +148,7 @@ class EngagementComputer(object):
                 self.mySQLPwd = ''
         # Place to hold all stats for one class
         self.classStats = {}
-        self.db = MySQLDB(host=self.dbHost, user=self.mySQLUser, passwd=self.mySQLPwd, db=dbName)
+        self.db = MySQLDB(host=self.dbHost, user=self.mySQLUser, passwd=self.mySQLPwd, db='Edx')
         
     def run(self):
         '''
@@ -187,6 +156,51 @@ class EngagementComputer(object):
         class is not a thread. Goes through every wanted course and every student
         within that course. Partitions student activity times into sessions, and
         does the accounting.
+        
+        Method uses one of two queries as source for session information:
+        one query is used when no course name is provided, and engagement
+        is thus computed for all courses, the alternative query is used
+        when computation is limited to a single course. Here is an explanation 
+        of the query, which creates this table: 
+
+             course_display_name, anon_screen_name, time, isVideo
+        
+        with one row for each event. The query with comments:
+        
+        -- 'select *' idiom needed to make MySQL use indexes in query optimizer 
+          SELECT *
+		  FROM  (
+	    -- Grab all events, creating binary 1/0 column 'isVideo' on the fly.
+	    -- With the latest approach of using mean time between events by one
+	    -- student as default duration for the last event in a session, this isVideo
+	    -- column is no longer relevant. It stays included in case we change our mind
+	    -- and wish to distinguish between video, and other event types after all:
+		  	       SELECT course_display_name,
+		  	              anon_screen_name,
+		  	              time,
+		  	              IF((event_type = 'load_video' OR 
+		  	                  event_type = 'play_video' OR 
+		  	                  event_type = 'pause_video' OR 
+		  	                  event_type = 'seek_video' OR 
+		  	                  event_type = 'speed_change_video'),1,0)
+		  	                 AS isVideo
+		  	       FROM Edx.EventXtract
+		  	    ) AS MainEvents
+		  	    
+		 -- Union these events with Forum events
+		  UNION ALL
+		  	    
+		  SELECT * 
+		  FROM  (
+		  	       SELECT course_display_name,\\
+		  	              EdxPrivate.idInt2Anon(forum_uid),
+		  	              created_at AS time,
+		  	              0 AS isVideo
+		  	       FROM EdxForum.contents
+		  	    ) AS ForumEvents
+		  GROUP BY course_display_name
+		  ORDER BY anon_screen_name, time;"            
+        
         '''
         self.studentSessionsDict   = {}
         # For saving all sessions for all students across all classes:
@@ -213,29 +227,50 @@ class EngagementComputer(object):
             queryEndTimeReported = False
             if self.courseToProfile is None:
                 # Profile all courses:
-                queryIterator = self.db.query('SELECT course_display_name, anon_screen_name, time, isVideo FROM %s ORDER BY course_display_name, anon_screen_name, time;' %
-                                              self.tableName)
+                mysqlCmd = "SELECT * \
+		  	                FROM  (\
+		  	                	     SELECT course_display_name, anon_screen_name, time, IF((event_type = 'play_video'),1,0) AS isVideo \
+		  	                	     FROM Edx.EventXtract \
+		  	                    UNION ALL \
+		  	                        SELECT course_display_name, EdxPrivate.idForum2Anon(forum_uid) AS anon_screen_name, created_at AS time, 0 AS isVideo \
+		  	                        FROM EdxForum.contents \
+		  	                      ) AS AllData\
+		  	                ORDER BY course_display_name, anon_screen_name, time;"              
             else:
-                queryIterator = self.db.query('SELECT course_display_name, anon_screen_name, time, isVideo FROM %s WHERE course_display_name = "%s" ORDER BY anon_screen_name, time;' %\
-                                              (self.tableName, self.courseToProfile))
+                mysqlCmd = "SELECT * \
+		  	                FROM  (\
+		  	                	     SELECT course_display_name, anon_screen_name, time, IF((event_type = 'play_video'),1,0) AS isVideo \
+		  	                	     FROM Edx.EventXtract \
+		  	                	     WHERE course_display_name = '%s'\
+		  	                    UNION ALL \
+		  	                        SELECT course_display_name, EdxPrivate.idForum2Anon(forum_uid) AS anon_screen_name, created_at AS time, 0 AS isVideo \
+		  	                        FROM EdxForum.contents \
+		  	                        WHERE course_display_name = '%s'\
+		  	                      ) AS AllData\
+		  	                ORDER BY course_display_name, anon_screen_name, time;" % (self.courseToProfile, self.courseToProfile)              
+                
+            queryIterator = self.db.query(mysqlCmd)
                 
             for activityRecord in queryIterator:
                 if not queryEndTimeReported:
                     self.log('Query done in %s' % str(datetime.timedelta(seconds=(time.time() - queryStartTime))))
+                    self.log('Beginning computation.')
                     queryEndTimeReported = True
                 currEvent = {'course_display_name' : activityRecord[COURSE_INDEX],
                              'anon_screen_name'    : activityRecord[STUDENT_INDEX],
                              'eventDateTime'       : activityRecord[TIME_INDEX], 
                              'isVideo'             : activityRecord[IS_VIDEO_INDEX]}
+                # Check whether it's a demo or sandbox course:
+                if self.filterCourses(currEvent):
+                    continue
+                
+                # Is this an invalid student?                
+                if self.filterStudents(currEvent['anon_screen_name']):
+                    continue
                 if prevEvent is None:
                     # First event of this course:
-                    # Check whether it's a demo or sandbox course:
-                    if self.filterCourses(currEvent):
-                        continue
-                    #... or a ghost student:
-                    if self.filterStudents(currEvent['anon_screen_name']):
-                        continue
-                    if len(currEvent['anon_screen_name']) > 0 and\
+                    if currEvent.get('anon_screen_name', None) is not None and\
+                       len(currEvent['anon_screen_name']) > 0 and\
                        len(currEvent['course_display_name']) > 0:
                         self.currStudent = currEvent['anon_screen_name']
                         self.currCourse  = currEvent['course_display_name']
@@ -263,27 +298,18 @@ class EngagementComputer(object):
                         self.log("Starting on course %s..." % currEvent['course_display_name'])
                     continue
 
-                # Is this an invalid student?                
-                if self.filterStudents(currEvent['anon_screen_name']):
-                    continue
                 if currEvent['course_display_name'] != self.currCourse:
                     # Previous event was last event of its course
-                    # Is this new event processable? I.e. does it have
-                    # all the info we need? If not, skip this event: 
-                    if self.filterCourses(currEvent):
-                        continue
-                    if len(currEvent['anon_screen_name']) > 0 and\
-                       len(currEvent['course_display_name']) > 0:
-                        # Account for the last session of current student in the current
-                        # class:
-                        self.wrapUpSession(self.currStudent, prevEvent['isVideo'], self.timeSpentThisSession, prevEvent['eventDateTime'])
-                        self.wrapUpCourse(self.currCourse, self.studentSessionsDict)
-                        # Start a new course:
-                        self.currStudent = currEvent['anon_screen_name']
-                        self.currCourse  = currEvent['course_display_name']
-                        self.sessionStartTime = currEvent['eventDateTime']
-                        prevEvent = currEvent
-                        self.log("Starting on course %s..." % self.currCourse)
+                    # Account for the last session of current student in the current
+                    # class:
+                    self.wrapUpSession(self.currStudent, prevEvent['isVideo'], self.timeSpentThisSession, prevEvent['eventDateTime'])
+                    self.wrapUpCourse(self.currCourse, self.studentSessionsDict)
+                    # Start a new course:
+                    self.currStudent = currEvent['anon_screen_name']
+                    self.currCourse  = currEvent['course_display_name']
+                    self.sessionStartTime = currEvent['eventDateTime']
+                    prevEvent = currEvent
+                    self.log("Starting on course %s..." % self.currCourse)
                     continue
                 # Steady state: Next event in same course as
                 # previous event:
@@ -519,23 +545,36 @@ class EngagementComputer(object):
         return True
         
     def filterStudents(self, anon_screen_name):
-        if anon_screen_name == "9c1185a5c5e9fc54612808977ee8f548b2258d31":
+        if anon_screen_name in ["9c1185a5c5e9fc54612808977ee8f548b2258d31", 
+                                'c8ced366_1048_4b4a_8e36_aa60f7b53dd8', 
+                                '-1', 
+                                '0',
+                                None]:
+            return True
+        if len(anon_screen_name) == 0:
             return True
         
+        return False
+        
     def filterCourses(self, currEvent):
-        if len(currEvent['course_display_name']) == 0:
+        course_display_name = currEvent['course_display_name']
+        if course_display_name is None:
             return True
-        if currEvent['anon_screen_name'] == '3c1276fa_c58b_43ef_bafa_d4d9f18bedf5' or currEvent['anon_screen_name'] == 'c8ced366_1048_4b4a_8e36_aa60f7b53dd8':
+        if len(course_display_name) == 0:
             return True
         # Catch all course names containing demo, sandbox, david,
         # and any space-including versions of the Education/EDUC115N/How_to_Learn_Math
         # course:
-        if EngagementComputer.FAKE_COURSE_PATTERN.search(currEvent['course_display_name']) is not None:
+        if EngagementComputer.FAKE_COURSE_PATTERN.search(course_display_name) is not None:
             return True 
-        if currEvent['course_display_name'] == 'Education/EDUC115N/How_to_Lean_Math':
+        if course_display_name == 'Education/EDUC115N/How_to_Lean_Math':
             return True
-        
-        
+        try:
+            int(course_display_name)
+            return True
+        except ValueError:
+            return False
+                
         
     def getCourseRuntime(self, courseName):
         '''
@@ -702,7 +741,7 @@ if __name__ == '__main__':
     parser.add_argument('years',
                         nargs='+',
                         type=int,
-                        help='A list of start years (YYYY) to limit the courses that are computed. Use All if all start years are acceptable'
+                        help='A list of start years (YYYY) to limit the courses that are computed. Use 0 if all start years are acceptable'
                         ) 
     
     
@@ -744,7 +783,7 @@ if __name__ == '__main__':
     else:
         courseName = args.course
     
-    if args.years == 'All':
+    if args.years == [0]:
         years = None
     else:
         years = args.years
@@ -757,11 +796,10 @@ if __name__ == '__main__':
         
     # -------------- Run the Computation ---------------
 
-    db = 'Misc'
     invokingUser = getpass.getuser()
     # Set mysql password to None, which will cause
     # the __init__() method to check ~/.ssh... 
-    comp = EngagementComputer(years, 'localhost', db, 'Activities', mySQLUser=invokingUser, mySQLPwd=None, courseToProfile=courseName)
+    comp = EngagementComputer(years, 'localhost', mySQLUser=invokingUser, mySQLPwd=None, courseToProfile=courseName)
     comp.run()
     
     # -------------- Output Results to Disk ---------------
